@@ -10,6 +10,8 @@
 ###############################################################################
 
 
+import copy
+import json
 import re
 from dataclasses import dataclass, field
 from itertools import combinations
@@ -31,6 +33,42 @@ class CheckResult:
     tree_data: Dict[str, Dict] = field(default_factory=dict)
 
 
+def checks_to_JSON(
+    checks_data,
+    all_check_results,
+    json_output_path=None,
+):
+    """
+    Serialise information about all checks into a JSON format
+    """
+    all_check_results_copy = copy.deepcopy(all_check_results)
+
+    result = {}
+    for job in all_check_results_copy:
+        result[job] = {}
+        for check in all_check_results_copy[job]:
+            result[job][check] = {}
+
+            result[job][check]["passed"] = all_check_results_copy[job][check].passed
+            result[job][check]["messages"] = all_check_results_copy[job][check].messages
+            result[job][check]["input"] = checks_data[check]
+            result[job][check]["output"] = all_check_results_copy[job][check].tree_data
+
+            # Temporary: remove histograms until a good way to serialise them is found
+            for tree in result[job][check]["output"]:
+                if "histograms" in result[job][check]["output"][tree]:
+                    hist_placeholder = ["placeholder"] * len(
+                        result[job][check]["output"][tree]["histograms"]
+                    )
+                    result[job][check]["output"][tree]["histograms"] = hist_placeholder
+
+    if json_output_path is not None:
+        with open(json_output_path, "w", encoding="utf8") as json_file:
+            json.dump(result, json_file, indent="  ")
+
+    return json.dumps(result, indent="  ")
+
+
 def num_entries(
     filepath_list,
     count,
@@ -107,7 +145,7 @@ def range_check(
     :param std_tolerance: Maximum shift tolerated between expected and observed values of standard deviation (optional)
     :param tree_pattern: A regular expression for the TTree object to check
     :returns: A CheckResult object, containing tree_data key/values:
-      * histogram: filled 1D histogram of the quantity defined by the expression parameter
+      * histograms: filled 1D histogram of the quantity defined by the expression parameter
       * num_entries: the total number of entries in the histogram (with blind ranges applied)
       * mean: the mean of the histogram (approximated using binned data)
       * variance: the variance of the histogram (approximated using binned data)
@@ -519,11 +557,11 @@ def range_check_bkg_subtracted(
     :param blind_ranges: regions to be blinded in the histogram
     :param tree_pattern: A regular expression for the TTree object to check
     :returns: A CheckResult object, containing tree_data key/values:
-      * histograms: a list of filled 1D histograms
-                    The first is the control variable used to perform the subtraction
-                    The second is the background-subtracted result
-      * hist_signal: filled 1D histogram of events in the signal window
-      * hist_background: filled 1D histogram of events in the background window
+      * histograms: a list of filled 1D histograms,
+                    Index 0: the control variable used to perform the subtraction
+                    Index 1: events in the signal window
+                    Index 2: events in the background window
+                    Index 3: the background-subtracted result
     """
     result = CheckResult("range_bkg_subtracted", True)
 
@@ -576,11 +614,12 @@ def range_check_bkg_subtracted(
                         signal_range["max"],
                     ]
 
+                    # Add signal & background region histograms (using same axis)
                     axis1 = hist.axis.Regular(
                         50, limits["min"], limits["max"], name=expression
                     )
-                    result.tree_data[key]["hist_signal"] = Hist(axis1)
-                    result.tree_data[key]["hist_background"] = Hist(axis1)
+                    result.tree_data[key]["histograms"].append(Hist(axis1))
+                    result.tree_data[key]["histograms"].append(Hist(axis1))
 
                 values_obj = {}
                 # Check if the branch is in the Tree or if the expressions are correctly written
@@ -664,8 +703,8 @@ def range_check_bkg_subtracted(
                     ]
 
                 # Fill signal & background histograms
-                result.tree_data[key]["hist_signal"].fill(test_array_sig)
-                result.tree_data[key]["hist_background"].fill(test_array_bkg)
+                result.tree_data[key]["histograms"][1].fill(test_array_sig)
+                result.tree_data[key]["histograms"][2].fill(test_array_bkg)
 
     # If no matches are found the check should be marked as failed, and can return early
     if len(result.tree_data) == 0:
@@ -677,8 +716,8 @@ def range_check_bkg_subtracted(
         for key in result.tree_data:
 
             # Require events in both signal and background histograms
-            if (result.tree_data[key]["hist_signal"].sum() == 0) or (
-                result.tree_data[key]["hist_background"].sum() == 0
+            if (result.tree_data[key]["histograms"][1].sum() == 0) or (
+                result.tree_data[key]["histograms"][2].sum() == 0
             ):
                 result.passed = False
                 result.messages += [
@@ -691,8 +730,8 @@ def range_check_bkg_subtracted(
 
             # Histogram subtraction
             hsub = (
-                result.tree_data[key]["hist_signal"]
-                + (-1 * alpha) * result.tree_data[key]["hist_background"]
+                result.tree_data[key]["histograms"][1]
+                + (-1 * alpha) * result.tree_data[key]["histograms"][2]
             )
             result.tree_data[key]["histograms"] += [hsub]
 
