@@ -127,6 +127,9 @@ CHECK_TYPE_SCHEMAS = {
         Optional("blind_ranges"): Map({"min": Float(), "max": Float()})
         | Seq(Map({"min": Float(), "max": Float()})),
     },
+    "branches_exist": {
+        "branches": Seq(Str()),
+    },
 }
 BASE_CHECK_SCHEMA = {
     "type": Enum(list(CHECK_TYPE_SCHEMAS)),
